#
# spec file for package glucat
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor main%{nil}

%define pname glucat

# Note: Blank flavor needs to build python bindings so that auto-generated python package names come out right
%if "%{flavor}" == "main"
%bcond_with    python
%define psuffix -main
%else
%bcond_without python
%define skip_python2 1
%endif

# Build failures when building doc for 15.4
%if (0%{?is_opensuse} && 0%{?sle_version} >= 150400)
%bcond_with pdfdoc
%else
%bcond_without pdfdoc
%endif

Name:           %{pname}%{?psuffix}
Version:        0.12.0
Release:        4.2
Summary:        Library of C++ templates implementing universal Clifford algebras
License:        LGPL-3.0-only
Group:          Development/Libraries/C and C++
URL:            http://glucat.sourceforge.net/
Source:         http://downloads.sourceforge.net/%{pname}/%{pname}-%{version}.tar.gz
# PATCH-FEATURE-OPENSUSE glucat-disable-doxygen-html-timestamp.patch badshah400@gmail.com -- Disable timestamps from html footer to make build reproducible
Patch0:         glucat-disable-doxygen-html-timestamp.patch
# PATCH-FIX-UPSTREAM glucat-include_dirs.patch paul.leopardi@anu.edu.au -- Add include_dirs to ext creation as per https://github.com/penguian/glucat/pull/53/files
Patch1:         glucat-include_dirs.patch
BuildRequires:  fdupes
BuildRequires:  gcc-c++
BuildRequires:  gcc-fortran
BuildRequires:  libboost_headers-devel
%if %{with python}
BuildRequires:  %{python_module Cython}
BuildRequires:  %{python_module numpy}
BuildRequires:  %{python_module setuptools}
BuildRequires:  python-rpm-macros
%python_subpackages
%else
BuildRequires:  doxygen
BuildRequires:  graphviz-gnome
%if %{with pdfdoc}
BuildRequires:  texlive-collection-fontsrecommended
BuildRequires:  texlive-latex-bin
BuildRequires:  texlive-metafont-bin
BuildRequires:  tex(adjustbox.sty)
BuildRequires:  tex(alphalph.sty)
BuildRequires:  tex(amsfonts.sty)
BuildRequires:  tex(auxhook.sty)
BuildRequires:  tex(bigintcalc.sty)
BuildRequires:  tex(bitset.sty)
BuildRequires:  tex(caption.sty)
BuildRequires:  tex(changepage.sty)
BuildRequires:  tex(collectbox.sty)
BuildRequires:  tex(colortbl.sty)
BuildRequires:  tex(courier.sty)
BuildRequires:  tex(enumitem.sty)
BuildRequires:  tex(etexcmds.sty)
BuildRequires:  tex(etoc.sty)
BuildRequires:  tex(etoolbox.sty)
BuildRequires:  tex(fancyhdr.sty)
BuildRequires:  tex(fancyvrb.sty)
BuildRequires:  tex(float.sty)
BuildRequires:  tex(geometry.sty)
BuildRequires:  tex(gettitlestring.sty)
BuildRequires:  tex(graphics.sty)
BuildRequires:  tex(hanging.sty)
BuildRequires:  tex(helvet.sty)
BuildRequires:  tex(hycolor.sty)
BuildRequires:  tex(hyperref.sty)
BuildRequires:  tex(ifoddpage.sty)
BuildRequires:  tex(iftex.sty)
BuildRequires:  tex(infwarerr.sty)
BuildRequires:  tex(intcalc.sty)
BuildRequires:  tex(kvdefinekeys.sty)
BuildRequires:  tex(kvoptions.sty)
BuildRequires:  tex(kvsetkeys.sty)
BuildRequires:  tex(letltxmacro.sty)
BuildRequires:  tex(listofitems.sty)
BuildRequires:  tex(ltxcmds.sty)
BuildRequires:  tex(multirow.sty)
BuildRequires:  tex(natbib.sty)
BuildRequires:  tex(newunicodechar.sty)
BuildRequires:  tex(pdfescape.sty)
BuildRequires:  tex(pdftexcmds.sty)
BuildRequires:  tex(refcount.sty)
BuildRequires:  tex(rerunfilecheck.sty)
BuildRequires:  tex(sectsty.sty)
BuildRequires:  tex(stackengine.sty)
BuildRequires:  tex(tabu.sty)
BuildRequires:  tex(tocloft.sty)
BuildRequires:  tex(ulem.sty)
BuildRequires:  tex(uniquecounter.sty)
BuildRequires:  tex(url.sty)
BuildRequires:  tex(varwidth.sty)
BuildRequires:  tex(wasysym.sty)
BuildRequires:  tex(xcolor.sty)
BuildRequires:  tex(xkeyval.sty)
BuildRequires:  tex(xtab.sty)
%endif
%endif

%description
GluCat is a library of template classes which model the universal
Clifford algebras over the field of real numbers, with arbitrary
dimension and arbitrary signature. GluCat implements a model of each
Clifford algebra corresponding to each non-degenerate quadratic form
up to a maximum number of dimensions.

%package -n %{pname}-devel
Summary:        Library of C++ templates implementing universal Clifford algebras
Group:          Development/Libraries/C and C++
Recommends:     %{name}-doc = %{version}

%description -n %{pname}-devel
GluCat is a library of template classes which model the universal
Clifford algebras over the field of real numbers, with arbitrary
dimension and arbitrary signature. GluCat implements a model of each
Clifford algebra corresponding to each non-degenerate quadratic form
up to a maximum number of dimensions.

This package contains the header files required for developing
applications using the glucat library.

%package -n %{pname}-doc
Summary:        Documentation for %{name}
Group:          Documentation/HTML

%description -n %{pname}-doc
GluCat is a library of template classes which model the universal
Clifford algebras over the field of real numbers, with arbitrary
dimension and arbitrary signature. GluCat implements a model of each
Clifford algebra corresponding to each non-degenerate quadratic form
up to a maximum number of dimensions.

This package provides the documentation for %{name}.

%prep
%autosetup -p1 -n %{pname}-%{version}

%build
%if %{with python}
%{python_expand # Apply to all supported python flavors
export PYTHON=$python
mkdir ../${PYTHON}_build
cp -pr ./ ../${PYTHON}_build
pushd ../${PYTHON}_build
%configure \
  --docdir=%{_docdir}/%{pname} \
  --enable-pyclical \
  --with-demo-dir=%{_docdir}/%{pname}-python%{$python_version}/demos

sed -i "s|-march=native||g" configure

%make_build clean all
popd
}
%else
%configure \
  --docdir=%{_docdir}/%{pname} \
  --disable-pyclical

sed -i "s|-march=native||g" configure

%make_build clean all
# Build doc only for main flavor
%make_build -C doc/ html %{?with_pdfdoc:doc} || (cat doc/api/latex/*.log ; false)
%endif

%install
%if %{with python}
%{python_expand #  all python flavors as configured above
export PYTHON=$python
pushd ../${PYTHON}_build
%make_install
# Remove non-python elements to be installed by main flavor
rm -fr %{buildroot}%{_includedir}/
popd
}
%else
%make_install
# Manually install doc files
mkdir -p %{buildroot}%{_docdir}/%{pname}
cp -pr doc/api/html %{buildroot}%{_docdir}/%{pname}/
%endif

# REMOVE FILES PKGED USING %%doc ANYWAY OR OTHERWISE NOT NEEDED
rm -fr %{buildroot}%{_docdir}/%{pname}/{AUTHORS,DESIGN,INSTALL,README,TODO}.md
rm -fr %{buildroot}%{_docdir}/%{pname}/{AUTHORS,DESIGN,INSTALL,README,TODO}
rm -fr %{buildroot}%{_docdir}/%{pname}/{COPYING,ChangeLog,NEWS,glucat.lsm}

%fdupes %{buildroot}%{_docdir}/%{pname}/html/

%if %{with python}
%check
%{python_expand #  all python flavors as configured above
export PYTHON=$python
pushd ../${PYTHON}_build
make %{?_smp_mflags} check
popd
}
%endif

%if %{without python}
%files -n %{pname}-devel
%license COPYING
%doc AUTHORS.md ChangeLog README.md TODO.md NEWS DESIGN.md
%{_includedir}/%{pname}/

%files -n %{pname}-doc
%dir %{_docdir}/%{pname}
%doc %{_docdir}/%{pname}/html/
%if %{with pdfdoc}
%doc doc/api/GluCat*.pdf
%endif

%else

%files %{python_files}
%{python_sitearch}/*
%{_docdir}/%{pname}-python%{python_version}/
%endif

%changelog
* Fri Dec 27 2024 Paul Leopardi <paul.leopardi@gmail.com>
- In setup_ext.py, in the call to Extension, Set include_dirs to
  a list obtained from all_includes.
- Add glucat-include_dirs.patch: upstream fix from
  https://github.com/penguian/glucat/pull/53
* Thu Mar 16 2023 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Add changepage.sty dependency, reenable PDF generation on TW
- Remove doxygen/graphviz-gnome build dependency for python
  build flavor, drop unused graphviz-gd.
- Fix some syntax errors in spec file
* Sun Mar 12 2023 Atri Bhattacharya <badshah400@gmail.com>
- Disable building PDF documentation for openSUSE >= 1550 and 15.4
  temporarily, until real issue with respect to dot errors is
  resolved.
* Wed Jun 15 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.12.0:
  * An improved sqrt() function that can deal with some cases
    where the matrix representation of multivector is a singular
    matrix.
  * The absolute and relative error estimation and approximate
    equality functions used in test11 have been streamlined and
    added to glucat/clifford_algebra.h and
    glucat/clifford_algebra_imp.h as the functions
    error_squared_tol(), error_squared() and approx_equal().
* Mon May  2 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.11.0:
  * Deprecated configuration options have been removed.
* Tue Mar 15 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.10.1:
  * Some GluCat configuration options have been deprecated.
* Mon Feb 14 2022 Atri Bhattacharya <badshah400@gmail.com>
- Split main and python builds into separate _multibuild flavors
  and build against all supported python versions for the blank
  flavor.
- Add glucat-disable-doxygen-html-timestamp.patch to disable
  timestamps on html footers in doxygen generated docs; this helps
  make builds reproducible.
* Mon Feb 14 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.10.0:
  * GluCat tuning is now done via a template parameter.
  * GluCat 0.10.0 introduces a change in the way that the headers
    are to be used.
  * The ./configure script has been refactored.
  * Successfully running ./configure --with-eig=bindings now
    requires a Fortran compiler.
  * The Python scripts used to build the PyClical extension module
    now use setuptools rather than distutils.
- New BuildRequires: gcc-fortran, python3-setuptools.
* Fri Dec 24 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.9.0:
  * Promotion of ScalarT now increases the number of floating
    point mantissa bits on all tested architectures, including ARM
    v8. Demotion is the reverse.
  * The default limit for the truncated() member function is now
    std::numericlimits<ScalarT>::epsilon().
  * The constructor from framedmulti<> to matrixmulti<> now uses
    the default truncation before the transform, and the
    constructor from matrixmulti<> to framedmulti<> uses the
    default truncation after the transform.
  * The truncation of output via operator<<() now depends on
    ostream precision(). See test17 for examples.
* Thu Oct 21 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.8.5:
  * Support for the the Intel icpx oneAPI C++ compiler.
  * Support for compiling with recent Clang, GCC and Intel C++
    compilers.
  * Restructured ./configure script to better support different
    configuration options and compiler differences.
  * Updated PyClical plotting demo to support more recent versions
    of Matplotlib Pyplot.
- New BuildRequires: tex(enumitem.sty) and tex(alphalph.sty)
  needed to build API documentation.
* Tue May 25 2021 Atri Bhattacharya <badshah400@gmail.com>
- Drop duplicated TeX related BuildRequires.
- Use `tex(foo.sty)` form for TeX BuildRequires consistently.
* Tue May 25 2021 Paul Leopardi <paul.leopardi@gmail.com>
- Fix builds for openSUSE > 1500 by adding appropriate
  BuildRequires needed to compile documentation against TeXLive
  2021.
* Sun Jul 12 2020 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 0.8.4:
  * Port to Python3.
  * Drop python2 support.
- Adapt to pyclical changes:
  * Re-enable pyclical extension for all openSUSE and drop no
    longer required compat macros.
  * Rename python-glucat pkg to python3-glucat and provide
    python-glucat.
  * Switch BuildRequires to python3 versions.
- Drop BuildRequires: python-devel, no longer required.
- Package DESIGN file in doc package.
* Sat Jan 11 2020 Atri Bhattacharya <badshah400@gmail.com>
- Disable python extension by default for openSUSE >= 1550 as it
  doesn't build against python3.
- Build PDF documentation for openSUSE > 1500 also; introduces
  new BuildRequires: tex(etoc.sty), tex(hanging.sty),
  tex(newunicodechar.sty), tex(stackengine.sty), tex(ulem.sty).
- Drop glucat-convert-demos-to-python3.patch: No longer needed
  since PDF documentation builds with previous change; drop
  libtool BuildRequires and don't run autoreconf before configure,
  both of which were only needed for this patch.
* Sun Feb 10 2019 Atri Bhattacharya <badshah400@gmail.com>
- Add glucat-disable-pdf-doc.patch to disable building pdf
  documentation until issues are fixed.
- Use %%%%license to install COPYING file.
* Tue Jul 10 2018 bwiedemann@suse.com
- make compilation independent of build system CPU (boo#1100520)
* Wed Feb  1 2017 adam.majer@suse.de
- use individual libboost-*-devel packages instead of boost-devel
* Mon Jul 11 2016 badshah400@gmail.com
- Update to version 0.8.2:
  + The GluCat classes basis_matrix_t and framed_multi<> have been
    refined.
  + The class basis_matrix_t is now based on int rather than on
    Scalar_T. This should make basis matrix generation more
    efficient, especially when Scalar_T is of class dd_real or
    qd_real.
  + The class framed_multi<> is now based on either
    map<index_set_t, Scalar_T> or unordered_map<index_set_t,
    Scalar_T>, rather than the previous base class, map<const
    index_set_t, Scalar_T> or unordered_map<const index_set_t,
    Scalar_T>.  This is for compatibility with the libc++
    implementations of these bases classes.
  + The framed_multi<> and matrix_multi<> constructors that use
    the parameters val, frm and prechecked have changed. The frame
    used for the constuction now includes the union of val.frame()
    with frm, and these constructors no longer throw an exception
    if val is oustide of frm.
  + A new function, reframe(lhs,rhs,lhs_reframed,rhs_reframed)
    fixes a bug in reframing for binary operations on class
    matrix_multi<>.
  + The function reframe(self, ixt) in pyclical/PyCliCal.pyx now
    puts self into a larger frame, containing the union of
    self.frame() and index set ixt.
  + GluCat now compiles with clang++ and libc++.
  + PyClical can now be built with g++ version 4.8 and possibly
    earlier versions.
  + GluCat also compiles cleanly with Intel C++ 16.0.2.
- Drop patches incorporated upstream:
  + glucat-pyclical-demos-dir.patch
  + glucat-doxygen-build-workaround.patch
  + glucat-pyclical-pythonpath.patch.
- Pyclical module now builds with gcc older than 4.9 again,
  reenable python-glucat for openSUSE 13.2 and older.
* Thu May  5 2016 badshah400@gmail.com
- Update to version 0.8.1:
  + The configuration option --with-tuning=arg controls test
    tuning via preprocessor macros and the header files
    test/tuning.h and test/undefine.h.
  + The configuration option --enable-check-isnan controls checks
    for NaN values, and the configuration option --with-pool-alloc
    controls the use of the Boost pool allocator.
  + The scripts test/test-all-config-options.sh,
    test/diff-all-config-outputs.sh and
    test/copy-all-config-outputs.sh enable comprehensive tesing of
    configuation options.
  + The use of Autotools has again been improved, with
    enhancements to make, make clean, make dist and make install,
    and the introduction of make check, make doc and make
    install-doc.
- Use make check for the %%check section.
- Add glucat-pyclical-demos-dir.patch to add a configure option
  "--with-demo-dir" to set a custom path to install the
  pyclical demos (defaults to %%%%{_datadir}/pyclical/demos);
  patch submitted upstream. We use this option to install the
  demos to %%{_docdir}/%%{name}/demos.
- Add glucat-doxygen-build-workaround.patch: required to
  circumvent a known doxygen 1.8.11 regression (bgo#762982); only
  required for openSUSE > 13.2. This patch should be dropped when
  the upstream doxygen bug is fixed and the fixed version is made
  available for openSUSE:Factory.
- Add glucat-pyclical-pythonpath.patch to include the correct dir
  for the Pycilcal module before building the pyclical notebooks;
  patch sent upstream.
- Add BuildRequires: automake, autoconf; new patches modify
  autotool build files.
- Package pyclical demos.
- Pyclical module does not build for openSUSE <= 13.2 or Leap
  42.1.
- Build HTML and PDF documentation, package them in a separate
  sub-package; add BuildRequires on texlive packages required for
  building the PDF documentation.
* Sun Oct 18 2015 schwab@suse.de
- Clean junk before building
* Thu Sep 10 2015 badshah400@gmail.com
- Update to version 0.8.0:
  + Drop all deprecated options (breaks backward compatibility):
  - <ext/hash_map> (was enabled by the option
    _GLUCAT_USE_GNU_CXX_HASH_MAP).
  - <backward/hash_map> (_GLUCAT_USE_BACKWARD_HASH_MAP).
  - <tr1/unordered_map> (_GLUCAT_USE_TR1_UNORDERED_MAP).
  - <tr1/random> (the old default: the option
    _GLUCAT_USE_STD_RANDOM is no longer needed, and is ignored,
    as <random> is now the default).
  - <alglib/evd.h> (_GLUCAT_USE_ALGLIB).
  - <boost/numeric/bindings/lapack/gees.hpp>
    (_GLUCAT_USE_BINDINGS_V1).
* Sat Sep  5 2015 badshah400@gmail.com
- Update to version 0.7.5:
  + The use of Autotools in configuring, building and installing
    GluCat and PyClical has been simplified and improved.
  + Deprecated configuation options have been dropped.
  + The default for --with-random is now std.
  + The default for --with-map is now stdunordered.
- Remove glucat-fix-32bit-build.patch: fixed upstream.
* Sat Dec 27 2014 badshah400@gmail.com
- Update to version 0.7.4:
  + Improved precision of the sqrt, log and exp functions.
- Workaround incorrect install location of pyclical module.
* Thu Apr  3 2014 badshah400@gmail.com
- Update to version 0.7.3:
  + Fixes a bug where the line print_exec("from PyClical import*")
    was left out of some of the pyclical_tutorial*.py files.
* Mon Mar 31 2014 badshah400@gmail.com
- Update to version 0.7.2:
  + When used with PyClical now generates IPython notebooks
  + The Pyclical tutorials are now accompanied by corresponding
    IPython notebooks.
  + The nork() function in matrix.h and matrix_imp.h is now
    accompanied by the signed_perm_nork() function, which is
    faster for the case used by fast<>() in matrix_mult_imp.h
  + Use of the Boost pool allocator with framed_multi<> is now
    optional, controlled via the new symbol
    _GLUCAT_USE_BOOST_POOL_ALLOC
  + GluCat compiles cleanly with g++ 4.8: does not trigger the
    "unused local typedefs" warning caused by the addition of
  - Wunused-local-typedefs to -Wall in g++ 4.8. See
    http://gcc.gnu.org/ml/gcc-patches/2011-07/msg02460.html
    http://gcc.gnu.org/ml/gcc-patches/2012-05/msg01388.html
  + Also compiles cleanly and builds correctly with clang++ 3.4
- Drop glucat-build-with-updated-boost.patch: incorporated
  upstream
- Implement a %%check section with included tests
- Drop conditionals solely relevant to end-of-life openSUSE
  versions
- Do not package unnecessary INSTALL file
- Update year in specfile COPYRIGHT.
* Thu Aug  1 2013 dvaleev@suse.com
- rename glucat-fix-i586-build.patch to glucat-fix-32bit-build.patch
- apply it for ppc 32Bit
* Sat Jun  8 2013 badshah400@gmail.com
- Add glucat-build-with-updated-boost.patch to fix building with
  boost > 1.50; patch came from upstream
- Include INSTALL file in package since it provides important
  information for compiling code built against glucat on
  openSUSE > 12.3.
* Tue Jan  8 2013 badshah400@gmail.com
- Update to version 0.7.1:
  + Fixes a bug which prevented compilation in the case where the
    class glucat::framed_multi<> is based on std::map.
* Thu Oct 25 2012 badshah400@gmail.com
- Add glucat-fix-i586-build.patch to fix build failures on i586.
* Thu Oct 11 2012 badshah400@gmail.com
- Update to version 0.7.0:
  + More accurate sqrt and transcendental functions and includes
    PyClical tutorials
  + The PyClical Python extension module now comes with a set of
    tutorials that guide you through working with Clifford algebra
    calculations in Python
  + The GluCat library now supports C++11, including
    <unordered_map> and <random>
  + Now has transcendental functions that are more accurate when
    used with higher precision arithmetic. This is accomplished by
    using different Pade' approximations according to
    the numerical precision needed
  + The class glucat::framed_multi<> based on hash_map or
    unordered_map now uses constructors that specify an initial
    number of buckets
  + The basis_element cache used for glucat::matrix_multi<> now
    uses pointers to matrices instead of matrices
  + The GluCat header files have been reorganized so that they now
    directly include the header files they depend on, rather than
    relying on glucat/glucat.h and glucat/glucat_imp.h to do this
  + The default options for ./configure now yield the simplest
    usable production case: ./configure without arguments with
    GluCat 0.7.0 gives the same results as
    "./configure --with-map=unordered --enable-debug=no
  - -enable-pyclical" did with GluCat 0.6.0
  + The README and INSTALL files have been completely rewritten,
    and AUTHORS has been updated.
* Sun May 13 2012 badshah400@gmail.com
- Update to version 0.6.0:
  + Includes the PyClical extension module for Python
  + Changes to the clifford_algebra<> API:
  - The real() function is now synonymous with scalar() and the
    imag() function always returns 0
  - New operators lhs | rhs and lhs |= rhs transform lhs by the
    twisted adjoint action of rhs
  - New member functions, grade() : maximum of the grades of
    each term;  pure() : pure part
  - New function complexifier(val) : square root of -1 which
    commutes with all members of the frame of the multivector
    val. The name "elliptic" is now deprecated: use
    "complexifier" instead
  - New functions sqrt(val,i,prechecked), and
    log(val,i,prechecked) : square root and logarithm of
    multivector with specified complexifier
- Split out python-glucat package with the python module
- Package is now arch dependent.
* Sun Mar 18 2012 scorot@free.fr
- add conditionnal noarch target to devel package to fix SLE build
* Thu Mar 24 2011 badshah400@gmail.com
- Initial package (version 0.5.1)
